/**
 * @description  公共方法
 * @author       朱琦
 * @date         2022/3/4
 */

/**
 * 过滤点位数据
 * @param {Array} data 数据
 * @param {String} id 键值名
 * @param {String} name 键值名 或 模板字符串
 * @param {String} lon 经度 键值名
 * @param {String} lat 纬度 键值名
 * @param {Array}  iconFilter 图标配置
 * @param {Object}  extend 扩展数据
 * @returns {Array} 过滤后的数据
 */
const filterPointData = (data, id, name, lon, lat, iconFilter, extend = {}) => {
    const list = []
    if (!data || data.length === 0) {
        return list
    }
    if (!RegExp(/\{\{(\w+)\}\}/).test(name)) {
        name = `{{${name}}}`
    }

    data.forEach(item => {
        if (item[lon] && item[lat]) {
            list.push({
                ...item,
                ...extend,
                id: item[id],
                name: getTemplateVal(name, item),
                path: [parseFloat(item[lon]), parseFloat(item[lat])],
                icon: getIconName(item, iconFilter)
            })
        }
    })
    return list
}

/**
 * 获取icon图标名
 * @param {Object} item 单项数据
 * @param {Array} iconFilter icon过滤规则
 * @returns icon name
 */
const getIconName = (item, iconFilter) => {
    let icon = null

    if (iconFilter && iconFilter.length) {
        const arr = [...iconFilter]
        const length = arr.length

        if (length == 1) {
            icon = arr[0]
        } else {
            let func = arr[length - 1]
            let newArr = arr.concat()
            newArr.pop()
            const keyArr = newArr.map(i => item[i])
            icon = func(keyArr)
        }
    }

    return icon
}

/**
 * 获取 模板字符串解析 结果
 * @param {String} template 模板字符串解析
 * @param {Object} data     数据
 * @returns {String} 解析结果
 */
const getTemplateVal = (template, data) => {
    let string = template
    if (template) {
        string = template.replace(/\{\{(\w+)\}\}/g, (match, key) => {
            return data[key];
        })
    }
    return string;
}

/**
 * 合并过滤数组对象
 * @param {Array} originalArray 原始数组
 * @param {Array} newArray      新数组
 * @param {Sting} key           对比的key值
 * @returns {Array} 合并过滤后的数组
 */
const concatArrayObject = (originalArray, newArray, key) => {
    const newArr = [...newArray]
    const originalArr = originalArray.map(item => {
        let obj = null
        const index = newArr.findIndex(i => i[key] === item[key])
        if (index > -1) {
            obj = newArr[index]
            newArr.splice(index, 1)
        }
        return {
            ...item,
            ...obj
        }
    })
    return originalArr.concat(newArr)
}

/**
 * 递归组件的所有子组件 执行方法并传参 仅vue2可用
 * @param {vue component} component vue 组件
 * @param {String} funName 方法名
 * @param {any} param 参数
 */
const getComponentChildren = (component, funName, param) => {
    try {
        component[funName] && component[funName](param);
        component.$children.forEach((chilComponent) => getComponentChildren(chilComponent, funName, param))
    } catch (e) {
        console.error("map load event,exception is " + e);
    }
}

const isValidPath = (path) => {
  if (!path || path.constructor !== Array || !path.length) return false

  if (path[0].constructor !== Array) {
    if (!Number(path[0]) || !Number(path[1])) return false
  }

  return true
}

export {
    filterPointData,
    getTemplateVal,
    concatArrayObject,
    getComponentChildren,
    getIconName,
    isValidPath
}